﻿using System;
using System.Collections.Generic;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace ArmoredCar
{
    internal class DrawningArmoredCar
    {
        /// <summary>
        /// Класс-сущность
        /// </summary>
        public EntityArmoredCar ArmoredCar { private set; get; }
        /// <summary>
        /// Левая координата отрисовки бронированной машины
        /// </summary>
        private float _startPosX;
        /// <summary>
        /// Верхняя кооридната отрисовки бронированной машины
        /// </summary>
        private float _startPosY;
        /// <summary>
        /// Ширина окна отрисовки
        /// </summary>
        private int? _pictureWidth = null;
        /// <summary>
        /// Высота окна отрисовки
        /// </summary>
        private int? _pictureHeight = null;
        /// <summary>
        /// Ширина отрисовки бронированной машины
        /// </summary>
        private readonly int _carWidth = 80;
        /// <summary>
        /// Высота отрисовки бронированной машины
        /// </summary>
        private readonly int _carHeight = 50;
        /// <summary>
        /// Инициализация свойств
        /// </summary>
        /// <param name="speed">Скорость</param>
        /// <param name="weight">Вес бронированной машины</param>
        /// <param name="bodyColor">Цвет кузова</param>
        public void Init(int speed, float weight, Color bodyColor)
        {
            ArmoredCar = new EntityArmoredCar();
            ArmoredCar.Init(speed, weight, bodyColor);
        }
        /// <summary>
        /// Установка позиции бронированной машины
        /// </summary>
        /// <param name="x">Координата X</param>
        /// <param name="y">Координата Y</param>
        /// <param name="width">Ширина картинки</param>
        /// <param name="height">Высота картинки</param>
        public void SetPosition(int x, int y, int width, int height)
        {
            
            if (x >= 0 && y >= 0 && x + _carWidth < width && y + _carHeight < height) {
                _startPosX = x;
                _startPosY = y;
                _pictureWidth = width;
                _pictureHeight = height; 
            }
        }

        /// <summary>
        /// Изменение направления перемещения
        /// </summary>
        /// <param name="direction">Направление</param>
        public void MoveTransport(Direction direction)
        {
            if (!_pictureWidth.HasValue || !_pictureHeight.HasValue)
            {
                return;
            }
            switch (direction)
            {
                // вправо
                case Direction.Right:
                    if (_startPosX + _carWidth + ArmoredCar.Step < _pictureWidth)
                    {
                        _startPosX += ArmoredCar.Step;
                    }
                    break;
                //влево
                case Direction.Left:
                    
                    if (_startPosX - ArmoredCar.Step > 0)
                    {
                        _startPosX -= ArmoredCar.Step;
                    }
                    break;
                //вверх
                case Direction.Up:
                    
                    if (_startPosY - ArmoredCar.Step > 0)
                    {
                        _startPosY -= ArmoredCar.Step;
                    }
                    break;
                //вниз
                case Direction.Down:
                    if (_startPosY + _carHeight + ArmoredCar.Step < _pictureHeight)
                    {
                        _startPosY += ArmoredCar.Step;
                    }
                    break;
            }
        }
        /// <summary>
        /// Отрисовка бронированной машины
        /// </summary>
        /// <param name="g"></param>
        public void DrawTransport(Graphics g)
        {
            if (_startPosX < 0 || _startPosY < 0
            || !_pictureHeight.HasValue || !_pictureWidth.HasValue)
            {
                return;
            }
            Pen pen = new(Color.Black);
            
            
            Brush br = new SolidBrush(ArmoredCar?.BodyColor ?? Color.Black);
            g.FillRectangle(br, _startPosX + 20, _startPosY, 40, 20);
            g.FillRectangle(br, _startPosX, _startPosY + 20, 80, 20);

            g.FillEllipse(br, _startPosX, _startPosY + 30, 20, 20);
            
            g.FillEllipse(br, _startPosX + 80 - 20, _startPosY + 30, 20, 20);
            
            g.FillRectangle(br, _startPosX + 15, _startPosY + 20, 60, 30);

            Brush brGray = new SolidBrush(Color.LightGray);
            g.FillEllipse(brGray, _startPosX, _startPosY + 30, 20, 20);
            g.FillEllipse(brGray, _startPosX + 30, _startPosY + 30, 20, 20);
            g.FillEllipse(brGray, _startPosX + 80 - 20, _startPosY + 30, 20, 20);
        }
        /// <summary>
        /// Смена границ формы отрисовки
        /// </summary>
        /// <param name="width">Ширина картинки</param>
        /// <param name="height">Высота картинки</param>
        public void ChangeBorders(int width, int height)
        {
            _pictureWidth = width;
            _pictureHeight = height;
            if (_pictureWidth <= _carWidth || _pictureHeight <= _carHeight)
            {
                _pictureWidth = null;
                _pictureHeight = null;
                return;
            }
            if (_startPosX + _carWidth > _pictureWidth)
            {
                _startPosX = _pictureWidth.Value - _carWidth;
            }
            if (_startPosY + _carHeight > _pictureHeight)
            {
                _startPosY = _pictureHeight.Value - _carHeight;
            }
        }

    }
}
